/*
   _===_ _           _                  ______ _     _             _ _ _
  |  ___| |         | |                 | ___ (_)   | |           (_) | |
  | |__ | | ___  ___| |_ _ __ ___ ______| |_/ /_  __| | ___  _   _ _| | | ___ _   _ _ __
  |  __|| |/ _ \/ __| __| '__/ _ \______| ___ \ |/ _` |/ _ \| | | | | | |/ _ \ | | | '__|
  | |___| |  __/ (__| |_| | | (_) |     | |_/ / | (_| | (_) | |_| | | | |  __/ |_| | |
  \____/|_|\___|\___|\__|_|  \___/      \____/|_|\__,_|\___/ \__,_|_|_|_|\___|\__,_|_|

Contrôle/Configuration de l'ADC externe ADS1113 pour le Fluke 8920A
Conçu pour rouler sur un Arduino-Nano 
Électro-Bidouilleur - Juin 2020

Format des données transmises sur le port série:
	"DDDD,G<LF>" :
		DDDD : Lecture de l'ADC en format décimal (0-32767)
		G : Gamme courante du Fluke 8920A (0-6):
			1 : 2 mV
			2: 20 mV
			3: 200 mV
			4 : 2 V
			5 : 20 V
			6 : 200 V
			7 : 700 V

Commandes recues sur le port série (exclure les guillemets):
	"a" : Place l'arduino en mode de lecture autonome et répétitive (mode par défaut au démarrage).
	"m" : Place l'arduino en mode de lecture manuelle (voir commande "r").
	"r" : Déclenche une lecture de l'ADC sur demande (uniquement en mode de lecture manuelle).

*/

#include <Wire.h>		// La librairie pour le port I2C

bool auto_read;
const unsigned char config_high = 0b00000101;	// Mots de configuration de l'ADC
const unsigned char config_low  = 0b00000000;    	// 000: 8 échantillons par seconde

#define F0 4			// Définition des trois broches des signaux de la gamme courante du 8920
#define F1 3
#define F2 2
#define adc_addr 72		// L'adresse I2C de l'ADC

// La fonction exécutée au démarrage de l'Arduino
void setup() 
{
  delay(2000); 			 // Petite pause de stabilisation du départ. Optionnelle.
 
  pinMode(F0, INPUT);    // Configuration des 3 broches de la gamme courante du 8920 en entrées
  pinMode(F1, INPUT);    
  pinMode(F2, INPUT);    

// Configuration et saisie du port série
  Serial.begin(115200);
  Serial.setTimeout(10000);

// Ouverture du port I2C 
  Wire.begin(); // Saisir le bus I2C connecté à l'ADC

// Configuration de l'ADC (I2C addresse 72)
  Wire.beginTransmission(adc_addr); 
  Wire.write(1); 				// Configurer le registre pointeur vers le registre de configuration (addresse 1)
  Wire.write(config_high); 		// Bits 15-8
  Wire.write(config_low); 		// Bits 7-0      
  Wire.endTransmission();		// Fin de transmission
  
  auto_read = true;				// Par défaut, l'Arduino fait des lectures automatiques de l'ADC
}

// La fonction qui lit la valeur de l'ADC ainsi que la gamme courante et les envoie au port série
void Read_ADC()
{
//Démarrer une conversion de l'ADC
  Wire.beginTransmission(adc_addr); 	// Début de transmission
  Wire.write(1);        				// Configurer le registre pointeur vers le registre de configuration (addresse 1)
  Wire.write(config_high | 10000000); 	// Démarrer une conversion (bit 15 mis à 1)
  Wire.write(config_low); 				// Bits 7-0 inchangés
  Wire.endTransmission();    			// Fin de transmission

// Attendre la fin de la conversion de l'ADC
  Wire.requestFrom(adc_addr, 1);					// demander 1 byte de l'ADC
  while (!((Wire.read() & 10000000)>>7))  Wire.requestFrom(adc_addr, 1);    // Attendre la fin de la conversion de l'ADC

// Générer le transfert du résultat de la conversion
  Wire.beginTransmission(adc_addr); 	// Début de transmission
  Wire.write(0);        				// Configurer le registre pointeur vers le registre de conversion (addresse 0)
  Wire.endTransmission();    			// Fin de transmission
  Wire.requestFrom(adc_addr, 2);    	// Demander 2 bytes de l'ADC (la lecture de l'ADC)

// Lire la valeur de la conversion et l'envoyer au port série
  Serial.print(Wire.read()<<8 | Wire.read(),DEC);	// Lire les 16 bits de la conversion et les envoyer au port série

// Lire la gamme courante du 8920 et l'envoyer au port série
  Serial.print(",");		// Ajouter une virgule comme séparateur
  Serial.print(digitalRead(F0)*4 + digitalRead(F1)*2 + digitalRead(F2)*1); // Envoyer la gamme (conversion binaire à décimal des trois bits de la gamme)
  Serial.print("\n"); 		// envoyer la fin de ligne (ASII 10) au port série
}

// La boucle principale de l'Arduino
void loop() 
{
unsigned char RxChar;			// Caractère lu du port série

  if (Serial.available()>0)     // Caractères disponibles dans le tampon de réception du port série?
  {     // Oui
    RxChar = Serial.read();     // Lecture d'un caractère
    if (RxChar == 'm') auto_read = false;		// Est-ce un caractère de mode manuel? Oui, change le mode a manuel
    else if (RxChar == 'a') auto_read = true;   // Est-ce un caractère de mode automatique? Oui, change le mode a automatique
	else if (RxChar == 'r' && auto_read == false) Read_ADC();    // Est-ce un caractère de lecture manuelle? Oui, fais une lecture manuelle
  }
  else if (auto_read == true) Read_ADC();		// Présentement en mode automatique? Oui, fais une lecture automatique
}

